package elements

import (
	"mover/gamedata"

	"github.com/hajimehoshi/ebiten/v2"
)

type Enemies interface {
	Update() error
	Draw()
	GetPosition() gamedata.Coordinates
	SetPosition(gamedata.Coordinates)
	SetTarget(gamedata.Coordinates)
	GetSprite() *ebiten.Image
	GetEnemyState() gamedata.EnemyState
	SetHit()
	SetToggle()
	IsToggled() bool
	SetTouched()
	ClearTouched()
	IsTouched() bool
	ExplosionInitiated() bool
	SetExplosionInitiated()
	Health() int
	MaxHealth() int
	GetAngle() float64
}
